@@ -1,16 +1,22 @@
-/* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
- * Copyright 1998-2004 Decklin Foster <decklin@red-bean.com>
- * This program is free software; see LICENSE for details. */
+/* aewm - An Exiguous Window Manager - vim:sw=4:ts=4:et
+ *
+ * Copyright 1998-2006 Decklin Foster <decklin@red-bean.com>. This
+ * program is free software; please see LICENSE for details. */
 
+#include <locale.h>
+#include <stdlib.h>
+#include <string.h>
+#include <stdio.h>
+#include <X11/Xatom.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
-#include "aeclients.h"
-#include "launch.h"
-#include "switch.h"
+#include "common.h"
+#include "atom.h"
+#include "menu.h"
 
-void make_launch_menu_item(menu_t, char *, char *);
-menu_t make_launch_sub_menu(menu_t, char *);
-void make_client_menu_item(unsigned char *, void *);
+void make_launch_menu_item(void *, char *, char *);
+void *make_launch_sub_menu(void *, char *);
+void make_client_menu_item(Window, GtkWidget *);
 void fork_exec_cb(GtkWidget *, char *);
 void raise_win_cb(GtkWidget *, Window);
 
@@ -19,25 +25,24 @@ int main(int argc, char **argv)
     GtkWidget *main_menu;
     int i, mode = LAUNCH;
     char *opt_config = NULL;
+    unsigned long read, left;
+    Window w;
 
+    setlocale(LC_ALL, "");
     gtk_init(&argc, &argv);
 
     for (i = 1; i < argc; i++) {
         if ARG("config", "rc", 1)  {
             opt_config = argv[++i];
-            continue;
-        }
-        if ARG("launch", "l", 0)   {
+        } else if ARG("launch", "l", 0)   {
             mode = LAUNCH;
-            continue;
-        }
-        if ARG("switch", "s", 0)   {
+        } else if ARG("switch", "s", 0)   {
             mode = SWITCH;
-            continue;
+        } else {
+            fprintf(stderr,
+                "usage: aemenu [--switch|-s] [--config|-rc <file>]\n");
+            exit(2);
         }
-        /* nothing matched */
-        fprintf(stderr, "usage: aemenu [--switch|-s] [--config|-rc <file>]\n");
-        exit(2);
     }
 
     main_menu = gtk_menu_new();
@@ -48,22 +53,15 @@ int main(int argc, char **argv)
     } else /* mode == SWITCH */ {
         dpy = GDK_DISPLAY();
         root = GDK_ROOT_WINDOW();
-        wm_state = XInternAtom(dpy,
-            "WM_STATE", False);
-        net_client_list = XInternAtom(dpy,
-            "_NET_CLIENT_LIST", False);
-        net_current_desktop = XInternAtom(dpy,
-            "_NET_CURRENT_DESKTOP", False);
-        net_wm_desktop = XInternAtom(dpy,
-            "_NET_WM_DESKTOP", False);
-        net_wm_state = XInternAtom(dpy,
-            "_NET_WM_STATE", False);
-        net_wm_state_skip_taskbar = XInternAtom(dpy,
-            "_NET_WM_STATE_SKIP_TASKBAR", False);
-        net_wm_state_skip_pager = XInternAtom(dpy,
-            "_NET_WM_STATE_SKIP_PAGER", False);
-        atom_foreach(root, net_client_list, XA_WINDOW,
-            make_client_menu_item, (void *)main_menu);
+        setup_switch_atoms();
+        for (i = 0, left = 1; left; i += read) {
+            read = get_atoms(root, net_client_list, XA_WINDOW, i,
+                &w, 1, &left);
+            if (read)
+                make_client_menu_item(w, main_menu);
+            else
+                break;
+        }
     }
 
     gtk_signal_connect_object(GTK_OBJECT(main_menu), "deactivate",
@@ -74,7 +72,7 @@ int main(int argc, char **argv)
     return 0;
 }
 
-void make_launch_menu_item(menu_t menu, char *label, char *cmd)
+void make_launch_menu_item(void *menu, char *label, char *cmd)
 {
     GtkWidget *item;
 
@@ -85,7 +83,7 @@ void make_launch_menu_item(menu_t menu, char *label, char *cmd)
     gtk_widget_show(item);
 }
 
-menu_t make_launch_sub_menu(menu_t menu, char *label)
+void *make_launch_sub_menu(void *menu, char *label)
 {
     GtkWidget *item, *new_menu;
 
@@ -98,15 +96,13 @@ menu_t make_launch_sub_menu(menu_t menu, char *label)
     return new_menu;
 }
 
-void make_client_menu_item(unsigned char *data, void *cb_data)
+void make_client_menu_item(Window w, GtkWidget *menu)
 {
-    Window w = *(Window *)data;
-    GtkWidget *menu = GTK_WIDGET(cb_data);
     GtkWidget *item;
     char buf[BUF_SIZE];
 
     if (is_on_cur_desk(w) && !is_skip(w)) {
-        get_wm_name(w, buf, sizeof buf);
+        snprint_wm_name(buf, sizeof buf, w);
         item = gtk_menu_item_new_with_label(buf);
         gtk_menu_append(GTK_MENU(menu), item);
         gtk_signal_connect(GTK_OBJECT(item), "activate",
